@@ -18,6 +18,7 @@
 ****************************************************************************/
 
 #include "fastsms.h"
+#include "ui_fastsms.h"
 #include "button.h"
 #include "dictionary.h"
 #include "busManagerServ.h"
@@ -26,6 +27,7 @@
 #include <QMessageAddress>
 #include <QMenuBar>
 #include <QMainWindow>
+//#include <QContactManager>
 
 #define STATE_SPACE 0
 #define STATE_SYM 1
@@ -41,6 +43,11 @@
 #define SETTING_ORIENTATION "orientation"
 #define SETTING_PHONEBOOK "phonebook"
 
+#define SETTING_FONTSIZE "fontsize"
+#define FONTSIZE_DEFAULT 0
+#define FONTSIZE_BIG 1
+#define FONTSIZE_BIGGER 2
+
 #define ORIENTATION_AUTO "auto"
 #define ORIENTATION_LANDSCAPE "landscape"
 #define ORIENTATION_PORTRAIT "portrait"
@@ -48,92 +55,95 @@
 #define PHONEBOOK_HORIZONTALCALL 0
 #define PHONEBOOK_MYCONTACTS 1
 
-#define BUTTON_CURNEXT "bt_cur_next"
-#define BUTTON_CURPREV "bt_cur_prev"
+#define BUTTON_CURNEXT "btMoveCurNext"
+#define BUTTON_CURPREV "btMoveCurPrev"
+
+#define ACTION_INS 0
+#define ACTION_DEL 1
+#define ACTION_SEND 1
 
 #define DATA_PATH "/data/"
 
+#define CAPITALIZATION_CHAR ".!?" //After those character, uppercase is needed
+#define IN_STYLE_CAPITALIZE 0
+#define IN_STYLE_UPPER 1
+#define IN_STYLE_LOWER 2
+#define IN_STYLE_DIGIT 3
+#define IN_STYLE_FW_ON 4
+#define IN_STYLE_FW_OFF 5
+
+#define SINGLE_CHAR_TIMEOUT 750
+
 //using namespace QtMobility;
 QTM_USE_NAMESPACE
 
-fastsms::fastsms(QWidget *parent) :
-    QWidget(parent)//,    ui(new Ui::fastsms)
+        Fastsms::Fastsms(QWidget *parent) :
+        QWidget(parent),
+        ui(new Ui::Fastsms)
 {
-    new busManagerServ(this);
-    registerService();
+    ui->setupUi(this);
+    setWindowTitle("FastSMS");
 
+    connect(ui->btOne, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btTwo, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btThree, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btFour, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btFive, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btSix, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btSeven, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btEight, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btNine, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btZero, SIGNAL(clicked()), this, SLOT(digitClicked()));
+    connect(ui->btTo, SIGNAL(clicked()), this, SLOT(openPhoneBook()));
+    connect(ui->btSend, SIGNAL(clicked()), this, SLOT(sendSMS()));
+    connect(ui->btMoveCurNext, SIGNAL(clicked()), this, SLOT(moveCursorBt()));
+    connect(ui->btMoveCurPrev, SIGNAL(clicked()), this, SLOT(moveCursorBt()));
+    connect(ui->btFunct, SIGNAL(clicked()), this, SLOT(deleteChar()));
+    connect(ui->btDelChar, SIGNAL(clicked()), this, SLOT(deleteChar()));
+    connect(ui->btNext, SIGNAL(clicked()), this, SLOT(nextWord()));
+    connect(ui->btPrev, SIGNAL(clicked()), this, SLOT(prevWord()));
 
-    //Design UI. TODO:use ui file
-    //ui->setupUi(this);
-    Button *to = createButton(tr("To:"),SLOT(openPhoneBook()));
-    Button *send = createButton(tr("Send"),SLOT(sendSMS()));
-    Button *btMoveCurNext = createButton(tr("->"),SLOT(moveCursorBt()));
-    btMoveCurNext->setObjectName(BUTTON_CURNEXT);
-    btMoveCurNext->setAutoRepeat(true);
-    Button *btMoveCurPrev = createButton(tr("<-"),SLOT(moveCursorBt()));
-    btMoveCurPrev->setObjectName(BUTTON_CURPREV);
-    btMoveCurPrev->setAutoRepeat(true);
-    Button *delChar = createButton(tr("C"),SLOT(deleteChar()));
-    delChar->setAutoRepeat(true);
-    Button *one = createButton(tr("1 .,!"),SLOT(digitClicked()));
-    Button *two = createButton(tr("2 abc"),SLOT(digitClicked()));
-    Button *three = createButton(tr("3 def"),SLOT(digitClicked()));
-    Button *four = createButton(tr("4 ghi"),SLOT(digitClicked()));
-    Button *five = createButton(tr("5 jkl"),SLOT(digitClicked()));
-    Button *six = createButton(tr("6 mno"),SLOT(digitClicked()));
-    Button *seven = createButton(tr("7 pqrs"),SLOT(digitClicked()));
-    Button *eight = createButton(tr("8 tuv"),SLOT(digitClicked()));
-    Button *nine = createButton(tr("9 wxyz"),SLOT(digitClicked()));
-    Button *zero = createButton(tr("0"),SLOT(digitClicked()));
-    Button *btNext = createButton(tr("Next"),SLOT(nextWord()));
-    btNext->setAutoRepeat(true);
-    Button *btPrev = createButton(tr("Prev"),SLOT(prevWord()));
-    btPrev->setAutoRepeat(true);
+    m_phoneNumber = ui->sPhoneNumber;
+    m_smsEdit = ui->sSmsEdit;
 
-    m_phoneNumber = new QLineEdit();
-    m_smsEdit = new QTextEdit();
+    //Add menu for functions button
+    QMenu *funcMenu = new QMenu("functions",this);
 
-    QGridLayout *queryLayout = new QGridLayout();
-    queryLayout->addWidget(to,0,0);
-    queryLayout->addWidget(m_phoneNumber,0,1);
-    queryLayout->addWidget(send,0,2);
-    queryLayout->addWidget(m_smsEdit,2,0,1,3);
+    QAction *aFastWrite = new QAction("Disable FastWrite",funcMenu);
+    aFastWrite->setData(IN_STYLE_FW_OFF);
+    connect(aFastWrite,SIGNAL(triggered()),this, SLOT(chageInputStyle()));
+    funcMenu->addAction(aFastWrite);
 
-    QGridLayout *keyboard = new QGridLayout();
-    keyboard->addWidget(btMoveCurPrev,0,0);
-    keyboard->addWidget(delChar,0,1);
-    keyboard->addWidget(btMoveCurNext,0,2);
-    keyboard->addWidget(one,1,0);
-    keyboard->addWidget(two,1,1);
-    keyboard->addWidget(three,1,2);
-    keyboard->addWidget(four,2,0);
-    keyboard->addWidget(five,2,1);
-    keyboard->addWidget(six,2,2);
-    keyboard->addWidget(seven,3,0);
-    keyboard->addWidget(eight,3,1);
-    keyboard->addWidget(nine,3,2);
-    keyboard->addWidget(btNext,4,0);
-    keyboard->addWidget(zero,4,1);
-    keyboard->addWidget(btPrev,4,2);
+    funcMenu->addSeparator();
 
-    QVBoxLayout *mainLayout = new QVBoxLayout(this);
+    QAction *aISCapitalized = new QAction("Abc",funcMenu);
+    aISCapitalized->setData(IN_STYLE_CAPITALIZE);
+    connect(aISCapitalized,SIGNAL(triggered()),this, SLOT(chageInputStyle()));
+    funcMenu->addAction(aISCapitalized);
 
-    mainLayout->addLayout(queryLayout);
-    mainLayout->addLayout(keyboard);
+    QAction *aISUpperCase = new QAction("ABC",funcMenu);
+    aISUpperCase->setData(IN_STYLE_UPPER);
+    connect(aISUpperCase,SIGNAL(triggered()),this, SLOT(chageInputStyle()));
+    funcMenu->addAction(aISUpperCase);
 
-    setLayout(mainLayout);    //End Design UI
+    QAction *aISLowerCase = new QAction("abc",funcMenu);
+    aISLowerCase->setData(IN_STYLE_LOWER);
+    connect(aISLowerCase,SIGNAL(triggered()),this, SLOT(chageInputStyle()));
+    funcMenu->addAction(aISLowerCase);
+
+    QAction *aISDigit = new QAction("123",funcMenu);
+    aISDigit->setData(IN_STYLE_DIGIT);
+    connect(aISDigit,SIGNAL(triggered()),this, SLOT(chageInputStyle()));
+    funcMenu->addAction(aISDigit);
+
+    ui->btFunct->setMenu(funcMenu);
+    //
 
     //load configuration
     m_dict = NULL;
     m_settings = new QSettings("it.rsc.fastsms","fastsms",this);
     loadDict();
 
-    //create dictionary
-//    QString sLang = m_settings->value(SETTING_LANGUAGE).toString();
-//    qDebug() << "Setting language:" << sLang;
-//    if(m_dict == NULL && !sLang.isEmpty())
-//        m_dict = new dictionary(new QString(m_settings->value(SETTING_LANGUAGE).toString()),this);
-
     //init variables
     m_nWordIndex=0;
     m_bNeedSave = false;
@@ -142,28 +152,46 @@ fastsms::fastsms(QWidget *parent) :
     m_nMsgLength = 0;
     m_nBeginWord = 0;
     m_nEndWord = 0;
+    m_nCharCount = 160;
+    m_nSMSCount = 1;
+    //init variables for capitalization
+    m_bMustCapitalize = true;
+    m_sCapitalization = CAPITALIZATION_CHAR;
+    m_nInputSyle = IN_STYLE_CAPITALIZE;
+    m_bFastInput = true;
+    m_bNewChar = true;
+    //init variables for standard input style
+    m_bTimerNewChar = true;
+    timerNewChar = new QTimer(this);
+    connect(timerNewChar,SIGNAL(timeout()),this,SLOT(alarmNewChar()));
+    timerNewChar->setSingleShot(true);
+    timerNewChar->setInterval(SINGLE_CHAR_TIMEOUT);
 
     //Menu
     QMenuBar *menu = new QMenuBar(this);
-    QMenu *fileMenu = new QMenu("File",menu);
+    QMenu *mainMenu = new QMenu("File",menu);
 
-    QAction *aOption = new QAction("Options",fileMenu);
+    QAction *aOption = new QAction("Options",mainMenu);
     connect(aOption,SIGNAL(triggered()),this, SLOT(openOptionDialog()));
-    fileMenu->addAction(aOption);
+    mainMenu->addAction(aOption);
 
-    QAction *aNewWord = new QAction("New Word",fileMenu);
+    QAction *aNewWord = new QAction("New Word",mainMenu);
     connect(aNewWord,SIGNAL(triggered()),this, SLOT(insertNewWord()));
-    fileMenu->addAction(aNewWord);
+    mainMenu->addAction(aNewWord);
 
-    QAction *aAbout = new QAction("About",fileMenu);
-    fileMenu->addAction(aAbout);
+    QAction *aDonate = new QAction("Donate",mainMenu);
+    connect(aDonate,SIGNAL(triggered()),this, SLOT(donate()));
+    mainMenu->addAction(aDonate);
+
+    QAction *aAbout = new QAction("About",mainMenu);
+    mainMenu->addAction(aAbout);
     connect(aAbout,SIGNAL(triggered()),this, SLOT(openAboutDialog()));
 
-    QAction *aQuit = new QAction("Quit",fileMenu);
+    QAction *aQuit = new QAction("Quit",mainMenu);
     connect(aQuit,SIGNAL(triggered()),this, SLOT(close()));
-    fileMenu->addAction(aQuit);
+    mainMenu->addAction(aQuit);
 
-    menu->addMenu(fileMenu);
+    menu->addMenu(mainMenu);
 
     //manage autorotate from start
     this->setFocus();
@@ -174,7 +202,7 @@ fastsms::fastsms(QWidget *parent) :
 }
 
 /** check if configuration and dict file exists. If not prompt to option dialog, else load it**/
-void fastsms::loadDict()
+void Fastsms::loadDict()
 {
     bool bOpenOpt = true;
 
@@ -190,15 +218,15 @@ void fastsms::loadDict()
 
         delete dictFile;
 
-        #if defined(Q_WS_MAEMO_5)
-            QString sCurrOrient = m_settings->value(SETTING_ORIENTATION).toString();
-            if(sCurrOrient.compare(ORIENTATION_LANDSCAPE) == 0)
-                setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
-            else if(sCurrOrient.compare(ORIENTATION_PORTRAIT) == 0)
-                setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
-            else
-                setAttribute(Qt::WA_Maemo5AutoOrientation, true);
-        #endif
+#if defined(Q_WS_MAEMO_5)
+        QString sCurrOrient = m_settings->value(SETTING_ORIENTATION).toString();
+        if(sCurrOrient.compare(ORIENTATION_LANDSCAPE) == 0)
+            setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
+        else if(sCurrOrient.compare(ORIENTATION_PORTRAIT) == 0)
+            setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
+        else
+            setAttribute(Qt::WA_Maemo5AutoOrientation, true);
+#endif
     }
 
     if(bOpenOpt)
@@ -208,13 +236,13 @@ void fastsms::loadDict()
 }
 
 /** Open and populate option dialog **/
-void fastsms::openOptionDialog()
+void Fastsms::openOptionDialog()
 {
     m_dialOptions = new QDialog(this);
     m_dialOptions->setAttribute(Qt::WA_DeleteOnClose);
     m_uiOptions.setupUi(m_dialOptions);
 
-    connect(m_uiOptions.save,SIGNAL(clicked()),this,SLOT(closeOptions()));
+    connect(m_uiOptions.btSave,SIGNAL(clicked()),this,SLOT(closeOptions()));
 
     QDir dictDir(QApplication::applicationDirPath() + "/data");
     QStringList dictList = dictDir.entryList(QStringList("*.idc"));
@@ -231,29 +259,32 @@ void fastsms::openOptionDialog()
         }
     }
 
-    m_uiOptions.orientationAuto->setChecked(true);
+    m_uiOptions.cOrientationAuto->setChecked(true);
 
     QString sCurrOrient = m_settings->value(SETTING_ORIENTATION).toString();
     if(sCurrOrient.compare(ORIENTATION_LANDSCAPE) == 0)
-        m_uiOptions.orientationLandscape->setChecked(true);
+        m_uiOptions.cOrientationLandscape->setChecked(true);
     else if(sCurrOrient.compare(ORIENTATION_PORTRAIT) == 0)
-        m_uiOptions.orientationPortrait->setChecked(true);
+        m_uiOptions.cOrientationPortrait->setChecked(true);
 
     int nPhoneBook = m_settings->value(SETTING_PHONEBOOK).toInt();
     m_uiOptions.cbPhonebook->setCurrentIndex(nPhoneBook);
 
+    m_uiOptions.cbFontSize->setCurrentIndex(m_settings->value(SETTING_FONTSIZE).toInt());
+
     m_dialOptions->show();
+
 }
 
 /** Open and populate option dialog **/
-void fastsms::openAboutDialog()
+void Fastsms::openAboutDialog()
 {
     QDialog * dlAbout = new QDialog(this);
     dlAbout->setAttribute(Qt::WA_DeleteOnClose);
 
     dlAbout->setWindowTitle("About");
     QGridLayout *layout = new QGridLayout();
-    QString sAbout = QString("Created by: Aimone Davide \n\n For requests,suggestions,bugs write to: fastsms@hotmail.it\n");
+    QString sAbout = QString("Created by: Aimone Davide \n\n For requests,suggestions,bugs write to: \n fastsms@hotmail.it \n");
     QLabel *msg = new QLabel(sAbout);
     layout->addWidget(msg,0,0);
     dlAbout->setLayout(layout);
@@ -262,7 +293,7 @@ void fastsms::openAboutDialog()
 }
 
 /** SLOT: close option dialog and save options in settings file **/
-void fastsms::closeOptions()
+void Fastsms::closeOptions()
 {
     QString sOldLang = m_settings->value(SETTING_LANGUAGE).toString();
 
@@ -270,26 +301,26 @@ void fastsms::closeOptions()
     m_settings->setValue(SETTING_LANGUAGE,sDictFile);
     m_settings->setValue(SETTING_PHONEBOOK,m_uiOptions.cbPhonebook->currentIndex());
 
-    if(m_uiOptions.orientationAuto->isChecked())
+    if(m_uiOptions.cOrientationAuto->isChecked())
     {
         m_settings->setValue(SETTING_ORIENTATION,ORIENTATION_AUTO);
-        #if defined(Q_WS_MAEMO_5)
-            setAttribute(Qt::WA_Maemo5AutoOrientation, true);
-        #endif
+#if defined(Q_WS_MAEMO_5)
+        setAttribute(Qt::WA_Maemo5AutoOrientation, true);
+#endif
     }
-    else if(m_uiOptions.orientationLandscape->isChecked())
+    else if(m_uiOptions.cOrientationLandscape->isChecked())
     {
         m_settings->setValue(SETTING_ORIENTATION,ORIENTATION_LANDSCAPE);
-        #if defined(Q_WS_MAEMO_5)
-            setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
-        #endif
+#if defined(Q_WS_MAEMO_5)
+        setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
+#endif
     }
-    else if(m_uiOptions.orientationPortrait->isChecked())
+    else if(m_uiOptions.cOrientationPortrait->isChecked())
     {
         m_settings->setValue(SETTING_ORIENTATION,ORIENTATION_PORTRAIT);
-        #if defined(Q_WS_MAEMO_5)
-            setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
-        #endif
+#if defined(Q_WS_MAEMO_5)
+        setAttribute(Qt::WA_Maemo5PortraitOrientation, true);
+#endif
     }
 
     if(sOldLang.compare(sDictFile) != 0)
@@ -302,24 +333,34 @@ void fastsms::closeOptions()
         m_dict = new dictionary(new QString(m_settings->value(SETTING_LANGUAGE).toString()),this);
     }
 
+    //Font size
+    m_settings->setValue(SETTING_FONTSIZE,m_uiOptions.cbFontSize->currentIndex());
+    int nFontSize = m_settings->value(SETTING_FONTSIZE).toInt();
+    QFont font = m_smsEdit->font();
+    //we will have 18,27 or 36
+    font.setPointSize(nFontSize * 9 + 18);
+    qDebug() << font.pointSize();
+    m_smsEdit->setFont(font);
+
     qDebug() << "Settings:" << m_settings->value(SETTING_LANGUAGE).toString() << "," << m_settings->value(SETTING_ORIENTATION).toString();
 
     m_dialOptions->close();
 }
 
-void fastsms::moveCursorBt()
+void Fastsms::moveCursorBt()
 {
     bool bMoveNext = false;
     m_nState = STATE_MOVECUR_BUTTON;
 
-    Button *clickedButton = qobject_cast<Button *>(sender());
+    //Button *clickedButton = qobject_cast<Button *>(sender());
+    QPushButton *clickedButton = qobject_cast<QPushButton *>(sender());
     QString btPressed = clickedButton->objectName();
     if(btPressed.compare(BUTTON_CURNEXT) == 0)
         bMoveNext = true;
 
     if(m_cursorSmsText.hasSelection() ||
-        (bMoveNext && m_cursorSmsText.atEnd()) ||
-        (!bMoveNext && m_cursorSmsText.atStart()))
+       (bMoveNext && m_cursorSmsText.atEnd()) ||
+       (!bMoveNext && m_cursorSmsText.atStart()))
     {
         m_smsEdit->disconnect();
 
@@ -375,7 +416,7 @@ void fastsms::moveCursorBt()
     }
 }
 
-void fastsms::cursorPosChanged()
+void Fastsms::cursorPosChanged()
 {
     if(m_nState != STATE_AFTERCURMOVED)
     {
@@ -420,7 +461,7 @@ void fastsms::cursorPosChanged()
     }
 }
 
-void fastsms::smsChanged()
+void Fastsms::smsChanged()
 {
     m_smsEdit->disconnect();
 
@@ -434,7 +475,7 @@ void fastsms::smsChanged()
 }
 
 /**Create button */
-Button *fastsms::createButton(const QString &text, const char *member)
+Button *Fastsms::createButton(const QString &text, const char *member)
 {
     Button *button = new Button(text,this);
     connect(button, SIGNAL(clicked()), this, member);
@@ -443,9 +484,11 @@ Button *fastsms::createButton(const QString &text, const char *member)
 }
 
 
-void fastsms::digitClicked()
+void Fastsms::digitClicked()
 {
-    Button *clickedButton = qobject_cast<Button *>(sender());
+
+    //Button *clickedButton = qobject_cast<Button *>(sender());
+    QPushButton *clickedButton = qobject_cast<QPushButton *>(sender());
     QString lastPress = clickedButton->text().at(0);
 
     if(m_phoneNumber->hasFocus())
@@ -456,68 +499,56 @@ void fastsms::digitClicked()
     {
         m_smsEdit->setFocus();
 
-        if(lastPress.compare("0") == 0)
+        //FastWrite input style selected
+        if(m_bFastInput && m_nInputSyle != IN_STYLE_DIGIT)
         {
-            m_nState = STATE_SPACE;
-            m_nMsgLength++;
-            m_sCompleteValue = lastPress;
-            m_nWordIndex=0;
-            m_lWord.clear();
-            m_lWord = m_dict->getWordList(&m_sCompleteValue);
-
-            m_nEndWord ++;
-            m_nBeginWord = m_nEndWord-1;
-            QString sChoosenWord = m_lWord.at(0);
-            addText(sChoosenWord);
-            m_sCompleteValue.clear();
-
-        }
-        else
-        {
-            if(m_nState == STATE_SPACE)
+            if(lastPress.compare("0") == 0)
             {
-                m_cursorSmsText.clearSelection();
-                m_nBeginWord = m_nEndWord;
-            }
-
-            m_sCompleteValue.append(lastPress);
-
-            if(lastPress.compare("1") == 0)
-                 m_nState = STATE_SYM;
-
-            if(m_dict->existsKey(&m_sCompleteValue))
-            {
-                m_nState = STATE_WORD;
+                m_nState = STATE_SPACE;
                 m_nMsgLength++;
+                m_sCompleteValue = lastPress;
                 m_nWordIndex=0;
+                m_lWord.clear();
                 m_lWord = m_dict->getWordList(&m_sCompleteValue);
 
-                m_nEndWord++;
+                m_nEndWord ++;
+                m_nBeginWord = m_nEndWord-1;
                 QString sChoosenWord = m_lWord.at(0);
-                updateText(sChoosenWord);
+                addText(sChoosenWord);
+                updateCharCounter(ACTION_INS);
+                m_sCompleteValue.clear();
+
             }
             else
             {
-                if(lastPress.compare("1") == 0)
+                if(m_nState == STATE_SPACE)
                 {
-                    m_nState = STATE_SYM;
-                    m_nMsgLength++;
-                    m_sCompleteValue.clear();
-                    m_sCompleteValue = lastPress;
+                    m_cursorSmsText.clearSelection();
+                    m_nBeginWord = m_nEndWord;
+                }
 
+                m_sCompleteValue.append(lastPress);
+
+                if(lastPress.compare("1") == 0)
+                    m_nState = STATE_SYM;
+
+                if(m_dict->existsKey(&m_sCompleteValue))
+                {
+                    m_nState = STATE_WORD;
+                    m_nMsgLength++;
                     m_nWordIndex=0;
                     m_lWord = m_dict->getWordList(&m_sCompleteValue);
 
-                    m_nBeginWord = m_nEndWord;
-                    m_nEndWord ++;
+                    m_nEndWord++;
                     QString sChoosenWord = m_lWord.at(0);
-                    addText(sChoosenWord);
+                    updateText(sChoosenWord);
+                    updateCharCounter(ACTION_INS);
                 }
                 else
                 {
-                    if(m_sCompleteValue.at(m_sCompleteValue.length()-2) == QChar('1'))//if(m_nState == STATE_SYM)
+                    if(lastPress.compare("1") == 0)
                     {
-                        m_nState = STATE_WORD;
+                        m_nState = STATE_SYM;
                         m_nMsgLength++;
                         m_sCompleteValue.clear();
                         m_sCompleteValue = lastPress;
@@ -529,21 +560,99 @@ void fastsms::digitClicked()
                         m_nEndWord ++;
                         QString sChoosenWord = m_lWord.at(0);
                         addText(sChoosenWord);
-
+                        updateCharCounter(ACTION_INS);
                     }
                     else
                     {
-                        //m_nState = STATE_UNKNOWN;
-                        m_sCompleteValue = m_sCompleteValue.left(m_sCompleteValue.length()-1);
-                        //TODO: no knowing words
+                        if(m_sCompleteValue.at(m_sCompleteValue.length()-2) == QChar('1'))//if(m_nState == STATE_SYM)
+                        {
+                            m_nState = STATE_WORD;
+                            m_nMsgLength++;
+                            m_sCompleteValue.clear();
+                            m_sCompleteValue = lastPress;
+
+                            m_nWordIndex=0;
+                            m_lWord = m_dict->getWordList(&m_sCompleteValue);
+
+                            m_nBeginWord = m_nEndWord;
+                            m_nEndWord ++;
+                            QString sChoosenWord = m_lWord.at(0);
+                            addText(sChoosenWord);
+                            updateCharCounter(ACTION_INS);
+                        }
+
+                        else
+                        {
+                            //m_nState = STATE_UNKNOWN;
+                            m_sCompleteValue = m_sCompleteValue.left(m_sCompleteValue.length()-1);
+                            //TODO: no knowing words
+                        }
                     }
                 }
             }
+        }//End FastWrite routine
+        else if(m_nInputSyle == IN_STYLE_DIGIT)//Only-Digit input selected
+        {
+            m_nState = STATE_SPACE;
+            m_nMsgLength++;
+            m_sCompleteValue = lastPress;
+            m_nWordIndex=0;
+            m_lWord.clear();
+            m_lWord = m_dict->getWordList(&m_sCompleteValue);
+
+            m_nEndWord ++;
+            m_nBeginWord = m_nEndWord-1;
+            addText(lastPress);
+            updateCharCounter(ACTION_INS);
+            m_sCompleteValue.clear();
         }
+        else if(!m_bFastInput) //standard-input selected
+        {
+
+            if(!m_bTimerNewChar && lastPress.compare(m_sLastPress) == 0)
+            {
+                m_bNewChar = false;
+
+                //Manage capitalization
+                int nPos = m_cursorSmsText.position();
+                nPos--;
+
+                if(m_bWasCapitalized)
+                    m_bMustCapitalize = true;
+                else
+                    m_bMustCapitalize = false;
+                //
+
+                nextWord();
+                timerNewChar->start();
+            }
+            else
+            {
+                m_bNewChar = true;
+                m_bWasCapitalized = false;
+                m_nState = STATE_SYM;
+                m_nMsgLength++;
+                m_sCompleteValue.clear();
+                m_sCompleteValue = lastPress;
+                m_nWordIndex=0;
+                m_lWord = m_dict->getWordList(&m_sCompleteValue,true);
+
+                m_nBeginWord = m_nEndWord;
+                m_nEndWord ++;
+                QString sChoosenWord = m_lWord.at(0);
+                addText(sChoosenWord);
+                updateCharCounter(ACTION_INS);
+
+                //reset timer
+                m_bTimerNewChar = false;
+                timerNewChar->start();
+            }
+        }
+        m_sLastPress = lastPress;
     }
 }
 
-void fastsms::nextWord()
+void Fastsms::nextWord()
 {
     if(m_lWord.count() > 0)
     {
@@ -556,7 +665,7 @@ void fastsms::nextWord()
     }
 }
 
-void fastsms::prevWord()
+void Fastsms::prevWord()
 {
     if(m_lWord.count() > 0)
     {
@@ -569,7 +678,7 @@ void fastsms::prevWord()
     }
 }
 
-void fastsms::insertNewWord()
+void Fastsms::insertNewWord()
 {
     m_dialNewWord = new QDialog(this);
     m_dialNewWord->setAttribute(Qt::WA_DeleteOnClose);
@@ -585,7 +694,7 @@ void fastsms::insertNewWord()
     m_dialNewWord->show();
 }
 
-void fastsms::newWordInserted()
+void Fastsms::newWordInserted()
 {
     QString *newWord = new QString(inputNewWord->text().toLower());
     if(!newWord->isEmpty())
@@ -603,38 +712,139 @@ void fastsms::newWordInserted()
         m_nWordIndex=m_lWord.indexOf(*newWord);
 
         updateText(*newWord);
+
+        for(int i=0; i<newWord->length();i++)
+        {
+            updateCharCounter(ACTION_INS);
+        }
     }
 
-     m_dialNewWord->close();
-    //m_bKnowWord=true;
+    m_dialNewWord->close();
 
     this->setFocus();
 }
 
+/** Update sms and character counter */
+void Fastsms::updateCharCounter(int action)
+{
+    if(action==ACTION_SEND)
+    {
+        m_nCharCount=160;
+        m_nSMSCount=1;
+    }
+    else if(action==ACTION_INS)
+    {
+        m_nCharCount--;
+
+        if(m_nCharCount == -1)
+        {
+            m_nSMSCount++;
+
+            if(m_nSMSCount==2)
+            {
+                m_nCharCount=145;
+            }
+            else if(m_nSMSCount >= 3)
+            {
+                m_nCharCount=152;
+            }
+        }
+    }
+    else if(m_nCharCount > -1)
+    {
+        m_nCharCount++;
+
+        if(m_nCharCount == 161)
+        {
+            m_nCharCount = 160;
+        }
+        else if(m_nCharCount == 153 && m_nSMSCount >= 3)
+        {
+            m_nCharCount = 0;
+            m_nSMSCount--;
+        }
+        else if(m_nCharCount == 146 && m_nSMSCount == 2)
+        {
+            m_nCharCount = 0;
+            m_nSMSCount--;
+        }
+    }
+
+    ui->lblCharCount->setText(QString::number(m_nCharCount,10));
+    ui->lblMessageCount->setText(QString::number(m_nSMSCount,10));
+}
+
 /** Delete cursors selection then insert sWord **/
-void fastsms::updateText(QString sWord)
+void Fastsms::updateText(QString sWord)
 {
     disconnect(m_smsEdit,SIGNAL(selectionChanged()),0,0);
 
-    m_cursorSmsText.insertText(sWord);
-
+    m_cursorSmsText.insertText(capitalizeIfNeeded(sWord));
     connect(m_smsEdit,SIGNAL(selectionChanged()),this,SLOT(cursorPosChanged()));
 
     qDebug() << "Write sms:" << m_smsEdit->toPlainText();
 }
 
 /** Add sWord in cursor current position, after deselect all **/
-void fastsms::addText(QString sWord)
+void Fastsms::addText(QString sWord)
 {
     disconnect(m_smsEdit,SIGNAL(selectionChanged()),0,0);
 
     m_cursorSmsText.clearSelection();
-    m_cursorSmsText.insertText(sWord);
+    m_cursorSmsText.insertText(capitalizeIfNeeded(sWord));
 
     connect(m_smsEdit,SIGNAL(selectionChanged()),this,SLOT(cursorPosChanged()));
 }
 
-void fastsms::deleteChar()
+QString Fastsms::capitalizeIfNeeded(QString sWord)
+{
+    if(m_nInputSyle == IN_STYLE_CAPITALIZE)
+    {
+        if(m_sCapitalization.contains(sWord))
+            m_bMustCapitalize = true;
+        else if(m_dict->isSymbol(sWord[0]))
+            m_bMustCapitalize = false;
+        else if(sWord==" ")
+            m_bMustCapitalize = false;
+
+
+        if(!m_bFastInput && m_bNewChar)
+        {
+            int nPos = m_cursorSmsText.position();
+            if(nPos > 0)
+            {
+                nPos--;
+
+                if(m_sCapitalization.contains(m_smsEdit->toPlainText().at(nPos)))
+                {
+                    sWord[0]=sWord[0].toUpper();
+                    m_bWasCapitalized = true;
+                }
+            }
+            else
+            {
+                sWord[0]=sWord[0].toUpper();
+                m_bWasCapitalized = true;
+            }
+        }
+        else if(m_bMustCapitalize)
+        {
+            sWord[0]=sWord[0].toUpper();
+        }
+
+
+    }
+    else if(m_nInputSyle == IN_STYLE_UPPER)
+        sWord = sWord.toUpper();
+    else if(m_nInputSyle == IN_STYLE_LOWER)
+        sWord = sWord.toLower();
+
+
+
+    return sWord;
+}
+
+void Fastsms::deleteChar()
 {
     if(m_phoneNumber->hasFocus())
     {
@@ -642,22 +852,23 @@ void fastsms::deleteChar()
     }
     else
     {
-            m_nState = STATE_DEL;
+        m_nState = STATE_DEL;
+        updateCharCounter(ACTION_DEL);
 
-            m_smsEdit->disconnect();
-            m_cursorSmsText.clearSelection();
-            m_cursorSmsText.deletePreviousChar();
+        m_smsEdit->disconnect();
+        m_cursorSmsText.clearSelection();
+        m_cursorSmsText.deletePreviousChar();
 
-            m_sCompleteValue.clear();
+        m_sCompleteValue.clear();
 
-            cursorPosChanged();
+        cursorPosChanged();
 
-            connect(m_smsEdit,SIGNAL(textChanged()),this,SLOT(smsChanged()));
-            connect(m_smsEdit,SIGNAL(selectionChanged()),this,SLOT(cursorPosChanged()));
+        connect(m_smsEdit,SIGNAL(textChanged()),this,SLOT(smsChanged()));
+        connect(m_smsEdit,SIGNAL(selectionChanged()),this,SLOT(cursorPosChanged()));
     }
 }
 
-void fastsms::openPhoneBook()
+void Fastsms::openPhoneBook()
 {
     bool ok = false;
     int Pb = m_settings->value(SETTING_PHONEBOOK).toInt();
@@ -683,7 +894,7 @@ void fastsms::openPhoneBook()
     }
 
     else if((Pb) == PHONEBOOK_HORIZONTALCALL)
-    {        
+    {
         QDBusMessage msg = QDBusMessage::createMethodCall(
                 "com.eitam.horizontalcall", // --dest
                 "/com/eitam/horizontalcall", // destination object path
@@ -698,13 +909,19 @@ void fastsms::openPhoneBook()
 
     if(!ok)
     {
+        QString sPhoneBook;
+        if((Pb) == PHONEBOOK_HORIZONTALCALL)
+            sPhoneBook = "Horizontal-Call";
+        else if (Pb == PHONEBOOK_MYCONTACTS)
+            sPhoneBook = "My-Contacts";
         QDialog *dlCont = new QDialog(this);
         dlCont->setAttribute(Qt::WA_DeleteOnClose);
 
         dlCont->setWindowTitle("Can't open selected phonebook");
         QGridLayout *layout = new QGridLayout();
-        QString sInfo = QString("\nYou need to install the selected phonebook to use this feature!!!\n"
-                                "You can change default phonebook in Option Menu.");
+        QString sInfo = QString("\nInstall " + sPhoneBook + "\n"
+                                "from Application Manager \n"
+                                "or change it in Option Menu.\n");
         QLabel *msg = new QLabel(sInfo);
         layout->addWidget(msg,0,0);
         dlCont->setLayout(layout);
@@ -712,7 +929,7 @@ void fastsms::openPhoneBook()
     }
 }
 
-void fastsms::sendSMS()
+void Fastsms::sendSMS()
 {
     QMessageService service;
     QMessageManager manager;
@@ -755,11 +972,15 @@ void fastsms::sendSMS()
     m_lWord.clear();
     m_nWordIndex=0;
 
+    //update char and sms count
+    updateCharCounter(ACTION_SEND);
+    //
+
     connect(m_smsEdit,SIGNAL(textChanged()),this,SLOT(smsChanged()));
     connect(m_smsEdit,SIGNAL(selectionChanged()),this,SLOT(cursorPosChanged()));
 }
 
-void fastsms::saveDictionaryOnExit()
+void Fastsms::saveDictionaryOnExit()
 {
     if(m_bNeedSave)
     {
@@ -767,7 +988,7 @@ void fastsms::saveDictionaryOnExit()
     }
 }
 
-bool fastsms::addRecipient(const QString &phone, const QString &name)
+bool Fastsms::addRecipient(const QString &phone, const QString &name)
 {
     m_phoneNumber->clear();
     m_phoneNumber->setText(phone);
@@ -778,7 +999,7 @@ bool fastsms::addRecipient(const QString &phone, const QString &name)
     return true;
 }
 
-bool fastsms::registerService()
+bool Fastsms::registerService()
 {
     QDBusConnection connection = QDBusConnection::sessionBus();
     if ( !connection.registerService("it.rsc.fastsms") ) {
@@ -793,27 +1014,61 @@ bool fastsms::registerService()
     return true;
 }
 
-void fastsms::doNothing()
+void Fastsms::chageInputStyle()
 {
+    QAction *action = qobject_cast<QAction *>(sender());
 
-}
-
-fastsms::~fastsms()
-{
-    //delete ui;
-    //delete m_sCompleteValue;
-}
-
-void fastsms::changeEvent(QEvent *e)
-{
-    QWidget::changeEvent(e);
-    switch (e->type())
+    if(action->data() == IN_STYLE_FW_OFF)
     {
-    case QEvent::LanguageChange:
-        //ui->retranslateUi(this);
-        break;
-    default:
-        break;
+        action->setText("Enable FastWrite");
+        action->setData(IN_STYLE_FW_ON);
+        m_bFastInput = false;
+        ui->lbl_fastWrite->setText("FastWrite OFF");
+    }
+    else if(action->data() == IN_STYLE_FW_ON)
+    {
+        action->setText("Disable FastWrite");
+        action->setData(IN_STYLE_FW_OFF);
+        m_bFastInput = true;
+        ui->lbl_fastWrite->setText("FastWrite ON");
+    }
+    else
+    {
+        m_nInputSyle = action->data().toInt();
+        ui->lbl_format->setText(action->text());
     }
 }
 
+void Fastsms::alarmNewChar()
+{
+    m_bTimerNewChar = true;
+}
+
+void Fastsms::donate()
+{
+    QDesktopServices::openUrl(QUrl("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=V353RBQ6TATN6"));
+}
+
+void Fastsms::doNothing()
+{
+
+
+}
+
+//void Fastsms::changeEvent(QEvent *e)
+//{
+//    QWidget::changeEvent(e);
+//    switch (e->type())
+//    {
+//    case QEvent::LanguageChange:
+//        //ui->retranslateUi(this);
+//        break;
+//    default:
+//        break;
+//    }
+//}
+
+Fastsms::~Fastsms()
+{
+    delete ui;
+}
